package com.jesuspinar.booksearch.service;

import com.jesuspinar.booksearch.dao.IBookDao;
import com.jesuspinar.booksearch.model.Book;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.PostConstruct;
import java.util.Arrays;
import java.util.Collection;
import java.util.function.Predicate;
import java.util.stream.Collectors;

@Service
public class BookQueryService implements IBookQueryService{

    @Autowired
    IBookDao iBookDao;

    private Predicate<Book> predicate;

    @PostConstruct
    public void init() {
        predicate = null;
    }

    @Override
    public Collection<Book> exec() {
        return iBookDao.findAll()
                .stream()
                .filter(predicate)
                .collect(Collectors.toList());
    }

    @Override
    public IBookQueryService anyGenre(String... genre) {
        Predicate<Book> pAnyGenre = (book -> Arrays.stream(genre).anyMatch(book.getGenre()::contains));
        predicate = (predicate == null) ? pAnyGenre : predicate.and(pAnyGenre);
        return this;
    }

    @Override
    public IBookQueryService allGenres(String... genres) {
        Predicate<Book> pAnyGenre = (book -> Arrays.stream(genres).allMatch(book.getGenre()::contains));
        predicate = (predicate == null) ? pAnyGenre : predicate.and(pAnyGenre);
        return this;
    }

    @Override
    public IBookQueryService anyTitle(String title) {
        Predicate<Book> pTitleContains  = (book -> book.getTitle().toLowerCase().contains(title.toLowerCase()));
        predicate = (predicate == null) ? pTitleContains : predicate.and(pTitleContains);

        return this;
    }
}
