package com.jesuspinar.booksearch.service;

import com.jesuspinar.booksearch.dao.IBookDao;
import com.jesuspinar.booksearch.model.Book;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Collection;
import java.util.List;
import java.util.stream.Collectors;


@Service
public class BookService implements IBookService{

    @Autowired
    IBookDao iBookDao;

    @Autowired
    IBookQueryService queryService;


    @Override
    public Collection<String> findAllGenres() {
        List<String> result = null;

        result = iBookDao.findAll()
                .stream()
                .map(book -> book.getGenre())
                .distinct()
                .sorted()
                .collect(Collectors.toList());

        return result;
    }

    @Override
    public Collection<Book> findByAnyGenre(String... genres) {
        return queryService.anyGenre(genres).exec();
    }

    @Override
    public Collection<Book> findByAllGenres(String... genres) {
        return queryService.allGenres(genres).exec();
    }

    @Override
    public Collection<Book> findByTitleContains(String title) {
        return queryService.anyTitle(title).exec();
    }

    @Override
    public Collection<Book> findAll() {
        return iBookDao.findAll();
    }
}
